@@ -7,8 +7,13 @@ package App::Ack::Resource::Basic;
 use warnings;
 use strict;
 
+use Fcntl ();
+use File::Spec ();
+
 use base 'App::Ack::Resource';
 
+use App::Ack;
+
 =head1 METHODS
 
 =head2 new( $filename )
@@ -27,17 +32,12 @@ sub new {
     my $self = bless {
         filename => $filename,
         fh       => undef,
-        opened   => undef,
+        opened   => 0,
     }, $class;
 
     if ( $self->{filename} eq '-' ) {
-        $self->{fh} = *STDIN;
-    }
-    else {
-        if ( !open( $self->{fh}, '<', $self->{filename} ) && $App::Ack::report_bad_filenames ) {
-            App::Ack::warn( "$self->{filename}: $!" );
-            return;
-        }
+        $self->{fh}     = *STDIN;
+        $self->{opened} = 1;
     }
 
     return $self;
@@ -53,6 +53,16 @@ sub name {
     return $_[0]->{filename};
 }
 
+sub basename {
+    my ( $self ) = @_;
+
+    # XXX definedness? pre-populate the slot with an undef?
+    unless ( exists $self->{basename} ) {
+        $self->{basename} = (File::Spec->splitpath($self->name))[2];
+    }
+
+    return $self->{basename};
+}
 
 =head2 $res->needs_line_scan( \%opts )
 
@@ -105,33 +115,13 @@ is true.
 sub reset {
     my $self = shift;
 
-    if( !seek( $self->{fh}, 0, 0 ) && $App::Ack::report_bad_filenames ) {
-        App::Ack::warn( "$self->{filename}: $!" );
+    # return if we haven't opened the file yet
+    if ( !defined($self->{fh}) ) {
+        return;
     }
 
-    return;
-}
-
-=head2 $res->next_text()
-
-API: Gets the next line of text from the resource.  Returns true
-if there is one, or false if not.
-
-Sets C<$_> with the line of text, and C<$.> for the ID number of
-the text.  This basically emulates a call to C<< <$fh> >>.
-
-=cut
-
-sub next_text {
-    if ( defined ($_ = readline $_[0]->{fh}) ) {
-        $. = ++$_[0]->{line};
-
-        # Normalize line endings if we need to
-        if ( /\r\n?$/ ) {
-            # Strip any trailing \r\n from the line and turn it to \n
-            s/[\r\n]+$/\n/;
-        }
-        return 1;
+    if( !seek( $self->{fh}, 0, 0 ) && $App::Ack::report_bad_filenames ) {
+        App::Ack::warn( "$self->{filename}: $!" );
     }
 
     return;
@@ -146,10 +136,17 @@ API: Close the resource.
 sub close {
     my $self = shift;
 
+    # return if we haven't opened the file yet
+    if ( !defined($self->{fh}) ) {
+        return;
+    }
+
     if ( !close($self->{fh}) && $App::Ack::report_bad_filenames ) {
         App::Ack::warn( $self->name() . ": $!" );
     }
 
+    $self->{opened} = 0;
+
     return;
 }
 
@@ -165,5 +162,39 @@ sub clone {
     return __PACKAGE__->new($self->name);
 }
 
+sub firstliney {
+    my ( $self ) = @_;
+
+    my $fh = $self->open();
+
+    if ( !exists $self->{firstliney} ) {
+        my $buffer = '';
+        my $rc     = sysread( $fh, $buffer, 250 );
+        unless($rc) { # XXX handle this better?
+            $buffer = '';
+        }
+        $buffer =~ s/[\r\n].*//s;
+        $self->{firstliney} = $buffer;
+        $self->reset;
+    }
+
+    $self->close;
+
+    return $self->{firstliney};
+}
+
+sub open {
+    my ( $self ) = @_;
+
+    return $self->{fh} if $self->{opened};
+
+    if ( ! open $self->{fh}, '<', $self->{filename} ) {
+        return;
+    }
+
+    $self->{opened} = 1;
+
+    return $self->{fh};
+}
 
 1;
