@@ -205,7 +205,7 @@ sub _options_block {
 
 # Clojure
 # http://clojure.org/
---type-add=clojure:ext:clj
+--type-add=clojure:ext:clj,cljs,edn,cljc
 
 # C
 # .xs are Perl C files
@@ -263,12 +263,16 @@ sub _options_block {
 # http://groovy.codehaus.org/
 --type-add=groovy:ext:groovy,gtmpl,gpp,grunit,gradle
 
+# GSP
+# http://groovy.codehaus.org/GSP
+--type-add=gsp:ext:gsp
+
 # Haskell
 # http://www.haskell.org/
 --type-add=haskell:ext:hs,lhs
 
 # HTML
---type-add=html:ext:htm,html
+--type-add=html:ext:htm,html,xhtml
 
 # Jade
 # http://jade-lang.com/
@@ -310,7 +314,7 @@ sub _options_block {
 
 # OCaml
 # http://caml.inria.fr/
---type-add=ocaml:ext:ml,mli
+--type-add=ocaml:ext:ml,mli,mll,mly
 
 # Matlab
 # http://en.wikipedia.org/wiki/MATLAB
@@ -384,6 +388,11 @@ sub _options_block {
 # http://learnboost.github.io/stylus/
 --type-add=stylus:ext:styl
 
+# Swift
+# https://developer.apple.com/swift/
+--type-add=swift:ext:swift
+--type-add=swift:firstlinematch:/^#!.*\bswift/
+
 # Tcl
 # http://www.tcl.tk/
 --type-add=tcl:ext:tcl,itcl,itk
@@ -412,7 +421,7 @@ sub _options_block {
 
 # XML
 # http://www.w3.org/TR/REC-xml/
---type-add=xml:ext:xml,dtd,xsl,xslt,ent
+--type-add=xml:ext:xml,dtd,xsd,xsl,xslt,ent,wsdl
 --type-add=xml:firstlinematch:/<[?]xml/
 
 # YAML
