@@ -139,6 +139,8 @@ sub uninvert_filter {
             $i--;
         }
     }
+
+    return;
 }
 
 
@@ -239,7 +241,7 @@ sub removed_option {
 
     $explanation ||= '';
     return sub {
-        warn "Option '$option' is not valid in ack 2\n$explanation";
+        warn "Option '$option' is not valid in ack 2.\n$explanation";
         exit 1;
     };
 }
@@ -249,10 +251,13 @@ sub get_arg_spec {
     my ( $opt, $extra_specs ) = @_;
 
     my $dash_a_explanation = <<'EOT';
-This is because we now have -k/--known-types which makes it only select files
-of known types, rather than any text file (which is the behavior of ack 1.x).
-You may have options in a .ackrc, or in the ACKRC_OPTIONS environment variable.
-Try using the --dump flag.
+You don't need -a, ack 1.x users.  This is because ack 2.x has
+-k/--known-types which makes it only select files of known types, rather
+than any text file (which is the behavior of ack 1.x).
+
+If you're surprised to see this message because you didn't put -a on the
+command line, you may have options in an .ackrc, or in the ACKRC_OPTIONS
+environment variable.  Try using the --dump flag to help find it.
 EOT
 
 =begin Adding-Options
@@ -278,12 +283,18 @@ EOT
 
 =cut
 
+    sub _context_value {
+        my $val = shift;
+
+        # Contexts default to 2.
+        return (!defined($val) || ($val < 0)) ? 2 : $val;
+    }
+
     return {
         1                   => sub { $opt->{1} = $opt->{m} = 1 },
-        'A|after-context=i' => \$opt->{after_context},
-        'B|before-context=i'
-                            => \$opt->{before_context},
-        'C|context:i'       => sub { shift; my $val = shift; $opt->{before_context} = $opt->{after_context} = ($val || 2) },
+        'A|after-context:-1'  => sub { shift; $opt->{after_context}  = _context_value(shift) },
+        'B|before-context:-1' => sub { shift; $opt->{before_context} = _context_value(shift) },
+        'C|context:-1'        => sub { shift; $opt->{before_context} = $opt->{after_context} = _context_value(shift) },
         'a'                 => removed_option('-a', $dash_a_explanation),
         'all'               => removed_option('--all', $dash_a_explanation),
         'break!'            => \$opt->{break},
@@ -690,6 +701,8 @@ sub check_for_mutually_exclusive_options {
             }
         }
     }
+
+    return;
 }
 
 
